window.Gleam.initSearch([{"doc":"shine_tree","title":"shine_tree","content":"# shine_tree\n\n[![Package Version](https://img.shields.io/hexpm/v/shine_tree)](https://hex.pm/packages/shine_tree)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/shine_tree/)\n\nA `FingerTree` implementation for general purpose functional programming in Gleam!\n\n```rs\npub opaque type ShineTree(u) {\n  Empty\n  Single(u)\n  Deep(count, Node(u), ShineTree(Node(u)), Node(u))\n}\n\ntype Node(u) {\n  One(u)\n  Two(u, u)\n  Three(u, u, u)\n  Four(u, u, u, u)\n}\n```\n\n## Usage\n\nTo install shine_tree into your project, use the following command.\n\n\n```sh\ngleam add shine_tree\n```\n\n```gleam\nimport shine_tree.{empty, push, to_list}\n\npub fn main() {\n  empty\n  |&gt; push(1)\n  |&gt; push(2)\n  |&gt; push(3)\n  |&gt; to_list\n  |&gt; io.debug\n  // [1, 2, 3]\n}\n```\n\nMost of the list functions in the std library have been implemented along with many more convenience methods like:\n\n- push\n- pop\n- shift\n- unshift\n- from_list\n- to_list\n- from_iterator\n- TODO: to_iterator\n- TODO: to_set\n- TODO: sort\n\nFurther documentation can be found at &lt;https://hexdocs.pm/shine_tree&gt;.\n\n## License\n\n```\nCopyright © 2024 Joshua Tenner &lt;tenner.joshua@gmail.com&gt;\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n```\n","url":"index.html"},{"doc":"shine_tree","title":"shine_tree","content":"|===+|===+   Compiled|===+ in 0.01s\n|===+|===+    Running|===+ benchmark.main\n\n\n## data set: Reduce 100 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.reduce(data) ...\nBenchmarking shine_tree.reduce ...\nCalculating statistics...\nFormatting results...\n\nName                        ips        average  deviation         median         99th %\nshine_tree.reduce        1.55 M      646.32 ns  ±7299.04%         500 ns        1400 ns\nlist.reduce(data)        1.08 M      924.88 ns  ±2632.60%         800 ns        1700 ns\n\nComparison: \nshine_tree.reduce        1.55 M\nlist.reduce(data)        1.08 M - 1.43x slower +278.55 ns\n\nMemory usage statistics:\n\nName                 Memory usage\nshine_tree.reduce            96 B\nlist.reduce(data)             0 B - 0.00x memory usage -96 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName              Reduction count\nshine_tree.reduce             267\nlist.reduce(data)             301 - 1.13x reduction count +34\n\n**All measurements for reduction count were the same**\n\n\n## data set: Reduce 10_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.reduce(data) ...\nBenchmarking shine_tree.reduce ...\nCalculating statistics...\nFormatting results...\n\nName                        ips        average  deviation         median         99th %\nshine_tree.reduce       18.06 K       55.37 μs   ±156.06%       51.60 μs       99.64 μs\nlist.reduce(data)       11.34 K       88.15 μs   ±653.22%       79.90 μs      141.38 μs\n\nComparison: \nshine_tree.reduce       18.06 K\nlist.reduce(data)       11.34 K - 1.59x slower +32.78 μs\n\nMemory usage statistics:\n\nName                 Memory usage\nshine_tree.reduce           192 B\nlist.reduce(data)             0 B - 0.00x memory usage -192 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName              Reduction count\nshine_tree.reduce         26.66 K\nlist.reduce(data)         30.00 K - 1.13x reduction count +3.34 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: Reduce 1_000_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.reduce(data) ...\nBenchmarking shine_tree.reduce ...\nCalculating statistics...\nFormatting results...\n\nName                        ips        average  deviation         median         99th %\nshine_tree.reduce        171.46        5.83 ms    ±14.25%        5.65 ms        9.25 ms\nlist.reduce(data)        117.34        8.52 ms    ±11.80%        8.26 ms       12.89 ms\n\nComparison: \nshine_tree.reduce        171.46\nlist.reduce(data)        117.34 - 1.46x slower +2.69 ms\n\nMemory usage statistics:\n\nName                 Memory usage\nshine_tree.reduce           320 B\nlist.reduce(data)             0 B - 0.00x memory usage -320 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName              Reduction count\nshine_tree.reduce          2.67 M\nlist.reduce(data)          3.00 M - 1.13x reduction count +0.33 M\n\n**All measurements for reduction count were the same**\n\n\n## data set: 100 prepends\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking [item, ..rest] * n ...\nBenchmarking shine_tree.unshift(Nil) * n ...\nCalculating statistics...\nFormatting results...\n\nName                                  ips        average  deviation         median         99th %\n[item, ..rest] * n                 1.24 M        0.81 μs  ±9806.83%        0.30 μs        1.10 μs\nshine_tree.unshift(Nil) * n        0.21 M        4.65 μs  ±1984.18%        1.80 μs        5.40 μs\n\nComparison: \n[item, ..rest] * n                 1.24 M\nshine_tree.unshift(Nil) * n        0.21 M - 5.77x slower +3.85 μs\n\nMemory usage statistics:\n\nName                           Memory usage\n[item, ..rest] * n                  1.56 KB\nshine_tree.unshift(Nil) * n        10.45 KB - 6.69x memory usage +8.89 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                        Reduction count\n[item, ..rest] * n                      101\nshine_tree.unshift(Nil) * n             403 - 3.99x reduction count +302\n\n**All measurements for reduction count were the same**\n\n\n## data set: 10_000 prepends\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking [item, ..rest] * n ...\nBenchmarking shine_tree.unshift(Nil) * n ...\nCalculating statistics...\nFormatting results...\n\nName                                  ips        average  deviation         median         99th %\n[item, ..rest] * n                16.45 K       60.78 μs   ±549.68%       43.60 μs      517.28 μs\nshine_tree.unshift(Nil) * n        2.57 K      389.51 μs    ±84.23%      308.10 μs     1000.62 μs\n\nComparison: \n[item, ..rest] * n                16.45 K\nshine_tree.unshift(Nil) * n        2.57 K - 6.41x slower +328.73 μs\n\nMemory usage statistics:\n\nName                           Memory usage\n[item, ..rest] * n                 0.153 MB\nshine_tree.unshift(Nil) * n         1.07 MB - 6.99x memory usage +0.91 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                        Reduction count\n[item, ..rest] * n                  15.89 K\nshine_tree.unshift(Nil) * n         44.84 K - 2.82x reduction count +28.94 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: 1_000_000 prepends\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking [item, ..rest] * n ...\nBenchmarking shine_tree.unshift(Nil) * n ...\nCalculating statistics...\nFormatting results...\n\nName                                  ips        average  deviation         median         99th %\n[item, ..rest] * n                  31.51       31.73 ms    ±11.35%       31.30 ms       43.49 ms\nshine_tree.unshift(Nil) * n          9.74      102.71 ms    ±22.54%       97.25 ms      193.04 ms\n\nComparison: \n[item, ..rest] * n                  31.51\nshine_tree.unshift(Nil) * n          9.74 - 3.24x slower +70.97 ms\n\nMemory usage statistics:\n\nName                           Memory usage\n[item, ..rest] * n                 15.26 MB\nshine_tree.unshift(Nil) * n       106.81 MB - 7.00x memory usage +91.55 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                                average  deviation         median         99th %\n[item, ..rest] * n                   1.13 M     ±2.79%         1.12 M         1.29 M\nshine_tree.unshift(Nil) * n          3.88 M     ±0.58%         3.87 M         3.97 M\n\nComparison: \n[item, ..rest] * n                   1.12 M\nshine_tree.unshift(Nil) * n          3.88 M - 3.44x reduction count +2.75 M\n\n\n## data set: from iterator of size 100\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking iterator.to_list ...\nBenchmarking shine_tree.from_iterator ...\nCalculating statistics...\nFormatting results...\n\nName                               ips        average  deviation         median         99th %\niterator.to_list              199.08 K        5.02 μs  ±1922.27%        2.70 μs        8.30 μs\nshine_tree.from_iterator       93.62 K       10.68 μs   ±967.63%        4.50 μs       21.00 μs\n\nComparison: \niterator.to_list              199.08 K\nshine_tree.from_iterator       93.62 K - 2.13x slower +5.66 μs\n\nMemory usage statistics:\n\nName                        Memory usage\niterator.to_list                12.76 KB\nshine_tree.from_iterator        24.23 KB - 1.90x memory usage +11.48 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                     Reduction count\niterator.to_list                  0.96 K\nshine_tree.from_iterator          1.35 K - 1.41x reduction count +0.39 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: from iterator of size 10_000\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking iterator.to_list ...\nBenchmarking shine_tree.from_iterator ...\nCalculating statistics...\nFormatting results...\n\nName                               ips        average  deviation         median         99th %\niterator.to_list                1.61 K      621.59 μs    ±65.94%      581.80 μs     1283.31 μs\nshine_tree.from_iterator        1.53 K      653.18 μs    ±45.18%      596.35 μs     1423.06 μs\n\nComparison: \niterator.to_list                1.61 K\nshine_tree.from_iterator        1.53 K - 1.05x slower +31.59 μs\n\nMemory usage statistics:\n\nName                        Memory usage\niterator.to_list                 1.16 MB\nshine_tree.from_iterator         2.35 MB - 2.03x memory usage +1.19 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                     Reduction count\niterator.to_list                100.01 K\nshine_tree.from_iterator        132.52 K - 1.33x reduction count +32.52 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: from iterator of size 1_000_000\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking iterator.to_list ...\nBenchmarking shine_tree.from_iterator ...\nCalculating statistics...\nFormatting results...\n\nName                               ips        average  deviation         median         99th %\niterator.to_list                  8.56      116.76 ms    ±12.35%      115.10 ms      164.14 ms\nshine_tree.from_iterator          6.12      163.44 ms    ±14.14%      158.05 ms      224.94 ms\n\nComparison: \niterator.to_list                  8.56\nshine_tree.from_iterator          6.12 - 1.40x slower +46.68 ms\n\nMemory usage statistics:\n\nName                        Memory usage\niterator.to_list               129.06 MB\nshine_tree.from_iterator       234.60 MB - 1.82x memory usage +105.54 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                             average  deviation         median         99th %\niterator.to_list                  9.31 M     ±0.40%         9.30 M         9.42 M\nshine_tree.from_iterator         12.79 M     ±0.40%        12.77 M        12.95 M\n\nComparison: \niterator.to_list                  9.30 M\nshine_tree.from_iterator         12.79 M - 1.37x reduction count +3.48 M\n\n\n## data set: append 100 items to 100 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.append(n) ...\nBenchmarking shine_tree.append(n) ...\nCalculating statistics...\nFormatting results...\n\nName                           ips        average  deviation         median         99th %\nshine_tree.append(n)        2.59 M      385.67 ns ±18622.04%         100 ns         400 ns\nlist.append(n)              2.48 M      402.54 ns ±10733.18%         200 ns         500 ns\n\nComparison: \nshine_tree.append(n)        2.59 M\nlist.append(n)              2.48 M - 1.04x slower +16.87 ns\n\nMemory usage statistics:\n\nName                    Memory usage\nshine_tree.append(n)         0.71 KB\nlist.append(n)               1.31 KB - 1.85x memory usage +0.60 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                 Reduction count\nshine_tree.append(n)              26\nlist.append(n)                     4 - 0.15x reduction count -22\n\n**All measurements for reduction count were the same**\n\n\n## data set: append 10_000 items to 10_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.append(n) ...\nBenchmarking shine_tree.append(n) ...\nCalculating statistics...\nFormatting results...\n\nName                           ips        average  deviation         median         99th %\nshine_tree.append(n)        1.02 M        0.98 μs  ±8899.05%        0.30 μs        1.20 μs\nlist.append(n)            0.0260 M       38.39 μs  ±1009.52%       15.80 μs      105.10 μs\n\nComparison: \nshine_tree.append(n)        1.02 M\nlist.append(n)            0.0260 M - 39.26x slower +37.41 μs\n\nMemory usage statistics:\n\nName                    Memory usage\nshine_tree.append(n)         1.88 KB\nlist.append(n)              49.78 KB - 26.55x memory usage +47.91 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                 Reduction count\nshine_tree.append(n)        0.0660 K\nlist.append(n)                3.99 K - 60.50x reduction count +3.93 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: append 1_000_000 items to 1_000_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.append(n) ...\nBenchmarking shine_tree.append(n) ...\nCalculating statistics...\nFormatting results...\n\nName                           ips        average  deviation         median         99th %\nshine_tree.append(n)      605.77 K     0.00165 ms  ±9251.18%     0.00050 ms     0.00150 ms\nlist.append(n)            0.0980 K       10.20 ms    ±94.71%        5.01 ms       38.33 ms\n\nComparison: \nshine_tree.append(n)      605.77 K\nlist.append(n)            0.0980 K - 6179.11x slower +10.20 ms\n\nMemory usage statistics:\n\nName                    Memory usage\nshine_tree.append(n)      0.00285 MB\nlist.append(n)               2.27 MB - 799.34x memory usage +2.27 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                         average  deviation         median         99th %\nshine_tree.append(n)        0.0950 K     ±0.00%       0.0950 K       0.0950 K\nlist.append(n)               85.08 K    ±28.02%        78.50 K       176.34 K\n\nComparison: \nshine_tree.append(n)        0.0950 K\nlist.append(n)               85.08 K - 895.55x reduction count +84.98 K\n\n\n## data set: calculate size of 100 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.length(n) ...\nBenchmarking shine_tree.size(n) ...\nCalculating statistics...\nFormatting results...\n\nName                         ips        average  deviation         median         99th %\nshine_tree.size(n)       30.25 M       33.06 ns ±10767.51%           0 ns         100 ns\nlist.length(n)            6.08 M      164.52 ns ±13749.17%         100 ns         200 ns\n\nComparison: \nshine_tree.size(n)       30.25 M\nlist.length(n)            6.08 M - 4.98x slower +131.46 ns\n\nMemory usage statistics:\n\nName                  Memory usage\nshine_tree.size(n)             0 B\nlist.length(n)                 0 B - 1.00x memory usage +0 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName               Reduction count\nshine_tree.size(n)               1\nlist.length(n)                   6 - 6.00x reduction count +5\n\n**All measurements for reduction count were the same**\n\n\n## data set: calculate size of 10_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.length(n) ...\nBenchmarking shine_tree.size(n) ...\nCalculating statistics...\nFormatting results...\n\nName                         ips        average  deviation         median         99th %\nshine_tree.size(n)       27.27 M      0.0367 μs ±14865.99%           0 μs       0.100 μs\nlist.length(n)          0.0593 M       16.87 μs   ±823.97%       15.70 μs       37.40 μs\n\nComparison: \nshine_tree.size(n)       27.27 M\nlist.length(n)          0.0593 M - 460.24x slower +16.84 μs\n\nMemory usage statistics:\n\nName                  Memory usage\nshine_tree.size(n)             0 B\nlist.length(n)                 0 B - 1.00x memory usage +0 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName               Reduction count\nshine_tree.size(n)               1\nlist.length(n)                 625 - 625.00x reduction count +624\n\n**All measurements for reduction count were the same**\n\n\n## data set: calculate size of 1_000_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.length(n) ...\nBenchmarking shine_tree.size(n) ...\nCalculating statistics...\nFormatting results...\n\nName                         ips        average  deviation         median         99th %\nshine_tree.size(n)       26.88 M     0.00004 ms  ±7448.78%           0 ms     0.00010 ms\nlist.length(n)         0.00040 M        2.52 ms   ±151.91%        2.09 ms        8.47 ms\n\nComparison: \nshine_tree.size(n)       26.88 M\nlist.length(n)         0.00040 M - 67687.73x slower +2.52 ms\n\nMemory usage statistics:\n\nName                  Memory usage\nshine_tree.size(n)             0 B\nlist.length(n)                 0 B - 1.00x memory usage +0 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName               Reduction count\nshine_tree.size(n)       0.00100 K\nlist.length(n)             62.50 K - 62500.00x reduction count +62.50 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: create range of 100 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.range(1, n) ...\nBenchmarking shine_tree.range(1, n) ...\nCalculating statistics...\nFormatting results...\n\nName                             ips        average  deviation         median         99th %\nshine_tree.range(1, n)      678.59 K        1.47 μs  ±8140.59%        0.40 μs        1.60 μs\nlist.range(1, n)            640.36 K        1.56 μs  ±4403.23%        0.80 μs           2 μs\n\nComparison: \nshine_tree.range(1, n)      678.59 K\nlist.range(1, n)            640.36 K - 1.06x slower +0.0880 μs\n\nMemory usage statistics:\n\nName                      Memory usage\nshine_tree.range(1, n)         2.10 KB\nlist.range(1, n)               1.56 KB - 0.74x memory usage -0.53906 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                   Reduction count\nshine_tree.range(1, n)             121\nlist.range(1, n)                   302 - 2.50x reduction count +181\n\n**All measurements for reduction count were the same**\n\n\n## data set: create range of 10_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.range(1, n) ...\nBenchmarking shine_tree.range(1, n) ...\nCalculating statistics...\nFormatting results...\n\nName                             ips        average  deviation         median         99th %\nshine_tree.range(1, n)       10.73 K       93.20 μs   ±140.38%       81.60 μs      219.81 μs\nlist.range(1, n)              8.81 K      113.55 μs   ±426.02%       97.70 μs      238.10 μs\n\nComparison: \nshine_tree.range(1, n)       10.73 K\nlist.range(1, n)              8.81 K - 1.22x slower +20.35 μs\n\nMemory usage statistics:\n\nName                      Memory usage\nshine_tree.range(1, n)       208.41 KB\nlist.range(1, n)             156.25 KB - 0.75x memory usage -52.15625 KB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                   Reduction count\nshine_tree.range(1, n)         15.10 K\nlist.range(1, n)               35.26 K - 2.33x reduction count +20.16 K\n\n**All measurements for reduction count were the same**\n\n\n## data set: create range of 1_000_000 items\n\nNot all of your protocols have been consolidated. In order to achieve the\nbest possible accuracy for benchmarks, please ensure protocol\nconsolidation is enabled in your benchmarking environment.\n\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9500 CPU @ 3.00GHz\nNumber of Available Cores: 2\nAvailable memory: 15.62 GB\nElixir 1.17.2\nErlang 27.0.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 4 s\nmemory time: 8 s\nreduction time: 4 s\nparallel: 2\ninputs: none specified\nEstimated total run time: 36 s\n\nBenchmarking list.range(1, n) ...\nBenchmarking shine_tree.range(1, n) ...\nCalculating statistics...\nFormatting results...\n\nName                             ips        average  deviation         median         99th %\nshine_tree.range(1, n)         28.67       34.88 ms    ±49.78%       35.85 ms       70.48 ms\nlist.range(1, n)               19.80       50.51 ms    ±25.35%       47.77 ms      122.66 ms\n\nComparison: \nshine_tree.range(1, n)         28.67\nlist.range(1, n)               19.80 - 1.45x slower +15.63 ms\n\nMemory usage statistics:\n\nName                      Memory usage\nshine_tree.range(1, n)        20.34 MB\nlist.range(1, n)              15.26 MB - 0.75x memory usage -5.08616 MB\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                           average  deviation         median         99th %\nshine_tree.range(1, n)          1.04 M     ±3.15%         1.03 M         1.15 M\nlist.range(1, n)                3.15 M     ±0.49%         3.14 M         3.21 M\n\nComparison: \nshine_tree.range(1, n)          1.03 M\nlist.range(1, n)                3.15 M - 3.02x reduction count +2.10 M\n","url":"benchmarks.html"},{"doc":"shine_tree","title":"shine_tree","content":"# Examples\n\nA simple FIFO work queue:\n\n```gleam\npub type Queue(item) {\n  Queue(items: FingerTree(item))\n}\n\npub fn new() {\n  shine_tree.empty\n}\n\npub fn enqueue(queue: Queue(item), item) {\n  Queue(shine_tree.push(queue.items, item))\n}\n\npub fn dequeue(queue: Queue(item)) {\n  case shine_tree.shift(queue.items) {\n    Error(_) -&gt; None\n    Ok(#(item, rest)) -&gt; Some(#(item, Queue(rest)))\n  }\n}\n","url":"examples.html"},{"doc":"shine_tree","title":"ShineTree","content":"pub opaque type ShineTree(u)\n An ordered tree of values used to implement other kinds of data\n structures like queues. Elements can be added or removed from the\n head or tail of the tree in nearly constant time.\n\nSynonyms:\nshine_tree.ShineTree\nshine_tree ShineTree","url":"shine_tree.html#ShineTree"},{"doc":"shine_tree","title":"empty","content":"pub const empty: ShineTree(a)\n\nSynonyms:\nshine_tree.empty\nshine_tree empty","url":"shine_tree.html#empty"},{"doc":"shine_tree","title":"all","content":"pub fn all(tree: ShineTree(a), f: fn(a) -&gt; Bool) -&gt; Bool\n Returns `true` if the predicate `f` returns `True` for each\n element in the ShineTree. Returns `False` otherwise.\n \n ## Examples\n\n \n shine_tree.all(shine_tree.empty, fn(x) { x == 0 })\n // -&gt; True\n \n\n \n all(shine_tree.from_list([-1, -42]), fn(x) { x &lt; 0 })\n // -&gt; True\n \n\n \n all(shine_tree.from_list([2, 4, 6, 8, 10, 11]), int.is_even)\n // -&gt; False\n \nSynonyms:\nshine_tree.all\nshine_tree all","url":"shine_tree.html#all"},{"doc":"shine_tree","title":"any","content":"pub fn any(tree: ShineTree(a), f: fn(a) -&gt; Bool) -&gt; Bool\n Returns `True` if the predicate `f` returns `True` for any\n element in the ShineTree. Returns `False` otherwise.\n \n ## Examples\n\n \n // No items in the tree will always be false\n any(shine_tree.empty, int.is_even)\n // -&gt; False\n \n\n \n any([10, 32, 42, 43, 44], fn(x) { x == 42 })\n // -&gt; True\n \n \n \n any([3, 4], fn(x) { x + 1 == 42 })\n // -&gt; False\n \nSynonyms:\nshine_tree.any\nshine_tree any","url":"shine_tree.html#any"},{"doc":"shine_tree","title":"append","content":"pub fn append(\n  tree_1: ShineTree(a),\n  tree_2: ShineTree(a),\n) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.append\nshine_tree append","url":"shine_tree.html#append"},{"doc":"shine_tree","title":"concat","content":"pub fn concat(trees: List(ShineTree(a))) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.concat\nshine_tree concat","url":"shine_tree.html#concat"},{"doc":"shine_tree","title":"contains","content":"pub fn contains(tree: ShineTree(a), u: a) -&gt; Bool\n\nSynonyms:\nshine_tree.contains\nshine_tree contains","url":"shine_tree.html#contains"},{"doc":"shine_tree","title":"count","content":"pub fn count(tree: ShineTree(a), where f: fn(a) -&gt; Bool) -&gt; Int\n Counts the number of elements in a given ShineTree satisfying a given predicate.\n\n This function has to traverse the entire tree to determine the number of elements,\n so it runs in linear time.\n \n ## Examples\n \n \n count(shine_tree.from_list([2, 3, 4, 5, 6, 7, 8, 9, 10]), int.is_even)\n // -&gt; 5\n \nSynonyms:\nshine_tree.count\nshine_tree count","url":"shine_tree.html#count"},{"doc":"shine_tree","title":"equals","content":"pub fn equals(a: ShineTree(a), b: ShineTree(a)) -&gt; Bool\n Compares two trees for equality. Since trees can have many shapes, the items\n can be in the correct order and values, but the tree is balanced differently.\n \n This function reduces over the two trees and compares their values, immediately\n returning `False` if they are not equal.\n \nSynonyms:\nshine_tree.equals\nshine_tree equals","url":"shine_tree.html#equals"},{"doc":"shine_tree","title":"filter","content":"pub fn filter(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; Bool,\n) -&gt; ShineTree(a)\n Creates a new tree containing all the elements of the given tree,\n for which the given predicate returns `True`.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; filter(int.is_odd)\n // -&gt; shine_tree.from_list([1, 3, 5, 7])\n \n \nSynonyms:\nshine_tree.filter\nshine_tree filter","url":"shine_tree.html#filter"},{"doc":"shine_tree","title":"fold_l","content":"pub fn fold_l(\n  over tree: ShineTree(a),\n  from v: b,\n  with f: fn(c, a) -&gt; c,\n) -&gt; c\n Reduces all the elements of the given tree into a single value,\n made by calling a given function on all the elements, traversing\n the tree from left to right.\n\n `fold_l(shine_tree.from_list([1, 2, 3]), 0, int.add)` is the equivalent of\n `add(add(add(0, 1), 2), 3)`.\n\n \n // calculate 20! (factorial)\n let n = 20\n shine_tree.fold_l(shine_tree.range(2, n), int.multiply)\n // -&gt; 2432902008176640000\n \nSynonyms:\nshine_tree.fold_l\nshine_tree fold_l","url":"shine_tree.html#fold_l"},{"doc":"shine_tree","title":"fold_r","content":"pub fn fold_r(tree: ShineTree(a), v: b, f: fn(c, a) -&gt; c) -&gt; c\n Reduces all the elements of the given tree into a single value,\n made by calling a given function on all the elements, traversing\n the tree from left to right.\n\n `fold_r(shine_tree.from_list([1, 2, 3]), 0, int.add)` is the equivalent of\n `add(add(add(0, 3), 2), 1)`.\n\n \n // calculate 20! (factorial)\n let n = 20\n shine_tree.fold_r(shine_tree.range(2, n), int.multiply)\n // -&gt; 2432902008176640000\n \nSynonyms:\nshine_tree.fold_r\nshine_tree fold_r","url":"shine_tree.html#fold_r"},{"doc":"shine_tree","title":"fold_until","content":"pub fn fold_until(\n  tree: ShineTree(a),\n  v: b,\n  with f: fn(b, a) -&gt; ContinueOrStop(b),\n) -&gt; b\n Fold a given value over the items of a given tree, starting with the beginning until the\n given function returns `Stop`.\n \n The accumulated value is then returned.\n \n ## Examples\n \n \n fold_until(shine_tree.from_list([2, 3, 4, 5, 6, 7, 8, 9, 10]), 0, fn(acc, x) {\n   case x {\n     5 -&gt; Stop(acc + 1)\n     _ -&gt; Continue(acc + x)\n   }\n })\n // -&gt; 10\nSynonyms:\nshine_tree.fold_until\nshine_tree fold_until","url":"shine_tree.html#fold_until"},{"doc":"shine_tree","title":"from_iterator","content":"pub fn from_iterator(iterable: Iterator(a)) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.from_iterator\nshine_tree from_iterator","url":"shine_tree.html#from_iterator"},{"doc":"shine_tree","title":"from_list","content":"pub fn from_list(values: List(a)) -&gt; ShineTree(a)\n Creates a new tree containing all the elements of the given list.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n // -&gt; It&#39;s a ShineTree with all the items in it!\n // Were you expecting a list?\n \n \nSynonyms:\nshine_tree.from_list\nshine_tree from_list","url":"shine_tree.html#from_list"},{"doc":"shine_tree","title":"map","content":"pub fn map(\n  tree: ShineTree(a),\n  with f: fn(a) -&gt; b,\n) -&gt; ShineTree(b)\n Maps all the elements of the given tree into a new tree where\n each element has been transformed by the given function.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.map(tree, int.multiply(2, _))\n // -&gt; shine_tree.from_list([2, 4, 6])\n \nSynonyms:\nshine_tree.map\nshine_tree map","url":"shine_tree.html#map"},{"doc":"shine_tree","title":"pop","content":"pub fn pop(tree: ShineTree(a)) -&gt; Result(#(a, ShineTree(a)), Nil)\n Pops an element from the end of the given tree.\n \n If there are no items, it returns `Error(Nil)`, otherwise\n it returns the popped item, and the resulting tree with the\n element removed.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.pop(tree)\n // -&gt; Ok(3, shine_tree.from_list([1, 2]))\n \n \n \n shine_tree.empty |&gt; shine_tree.pop\n // -&gt; Error(Nil)\n \nSynonyms:\nshine_tree.pop\nshine_tree pop","url":"shine_tree.html#pop"},{"doc":"shine_tree","title":"prepend","content":"pub fn prepend(\n  tree_1: ShineTree(a),\n  tree_2: ShineTree(a),\n) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.prepend\nshine_tree prepend","url":"shine_tree.html#prepend"},{"doc":"shine_tree","title":"push","content":"pub fn push(tree: ShineTree(a), value: a) -&gt; ShineTree(a)\n Pushes an element to the end of the given tree.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.push(tree, 4)\n // -&gt; shine_tree.from_list([1, 2, 3, 4])\n \nSynonyms:\nshine_tree.push\nshine_tree push","url":"shine_tree.html#push"},{"doc":"shine_tree","title":"range","content":"pub fn range(start: Int, finish: Int) -&gt; ShineTree(Int)\n Creates a tree of `n` consecutive integers, starting from `start`.\n and finishing with `finish`.\n \n ## Examples\n \n \n let ten_items = shine_tree.range(1, 10)\n \n \nSynonyms:\nshine_tree.range\nshine_tree range","url":"shine_tree.html#range"},{"doc":"shine_tree","title":"reverse","content":"pub fn reverse(tree: ShineTree(a)) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.reverse\nshine_tree reverse","url":"shine_tree.html#reverse"},{"doc":"shine_tree","title":"shift","content":"pub fn shift(\n  tree: ShineTree(a),\n) -&gt; Result(#(a, ShineTree(a)), Nil)\n Pops an element from the front of the given tree.\n \n If there are no items, it returns `Error(Nil)`, otherwise\n it returns the popped item, and the resulting tree with the\n element removed.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.shift(tree)\n // -&gt; Ok(1, shine_tree.from_list([2, 3]))\n \n \n \n shine_tree.empty |&gt; shine_tree.pop\n // -&gt; Error(Nil)\n \nSynonyms:\nshine_tree.shift\nshine_tree shift","url":"shine_tree.html#shift"},{"doc":"shine_tree","title":"single","content":"pub fn single(u: a) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.single\nshine_tree single","url":"shine_tree.html#single"},{"doc":"shine_tree","title":"size","content":"pub fn size(tree: ShineTree(a)) -&gt; Int\n\nSynonyms:\nshine_tree.size\nshine_tree size","url":"shine_tree.html#size"},{"doc":"shine_tree","title":"to_iterator","content":"pub fn to_iterator(tree: ShineTree(a)) -&gt; Iterator(a)\n Creates an iterator from a given tree, yielding each element in succession until\n there are no elements left.\n\n ## Examples\n\n \n let iter = unfold(shine_tree.from_list([1, 2, 3]))\n \n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(1, Iterator(Int))\n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(2, Iterator(Int))\n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(3, Iterator(Int))\n let a = iterator.step(iter)\n // -&gt; Done\n \n\nSynonyms:\nshine_tree.to_iterator\nshine_tree to_iterator","url":"shine_tree.html#to_iterator"},{"doc":"shine_tree","title":"to_list","content":"pub fn to_list(tree: ShineTree(a)) -&gt; List(a)\n Creates a new list containing all the elements of the given tree.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; to_list\n // -&gt; [1, 2, 3, 4, 5, 6, 7]\n \n \nSynonyms:\nshine_tree.to_list\nshine_tree to_list","url":"shine_tree.html#to_list"},{"doc":"shine_tree","title":"unshift","content":"pub fn unshift(tree: ShineTree(a), value: a) -&gt; ShineTree(a)\n Pushes an element to the beginning of the given tree.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.unshift(tree, 0)\n // -&gt; shine_tree.from_list([0, 1, 2, 3])\n \nSynonyms:\nshine_tree.unshift\nshine_tree unshift","url":"shine_tree.html#unshift"},{"doc":"shine_tree","title":"shine_tree","content":"","url":"shine_tree.html"}]);